@@ -8,8 +8,7 @@ and density compensation factors.
 """
 from sigpy.mri import app, linop
 
-from sigpy.mri import bloch, dcf, precond, samp, sim, util
-from sigpy.mri.bloch import *  # noqa
+from sigpy.mri import dcf, precond, samp, sim, util
 from sigpy.mri.dcf import *  # noqa
 from sigpy.mri.precond import *  # noqa
 from sigpy.mri.samp import *  # noqa
@@ -17,7 +16,6 @@ from sigpy.mri.sim import *  # noqa
 from sigpy.mri.util import *  # noqa
 
 __all__ = ['app', 'linop']
-__all__.extend(bloch.__all__)
 __all__.extend(dcf.__all__)
 __all__.extend(precond.__all__)
 __all__.extend(samp.__all__)
